@@ -1,6 +1,8 @@
 #   ifndef		DOC_SELECT_H
 #   define		DOC_SELECT_H
 
+#   include		"docLayoutPosition.h"
+
 /************************************************************************/
 /*									*/
 /*  Forward declarations of structs.					*/
@@ -11,23 +13,6 @@ struct BufferDocument;
 struct BufferItem;
 struct DocumentField;
 
-/************************************************************************/
-/*									*/
-/*  Used to layout lines of text.					*/
-/*									*/
-/*  A0 paper is 4* 30 cm high: 28.3465* 120* 20= 68031.6 Twips.		*/
-/*  So an unsigned short limits us to A1 paper.				*/
-/*									*/
-/************************************************************************/
-
-typedef struct LayoutPosition
-    {
-    unsigned short	lpPageYTwips;
-    unsigned short	lpPage;
-    unsigned char	lpColumn;
-    unsigned char	lpAtTopOfColumn;
-    } LayoutPosition;
-
 /************************************************************************/
 /*									*/
 /*  A position in a document.						*/
@@ -42,7 +27,7 @@ typedef struct LayoutPosition
 /*	array of notes in the section.					*/
 /*									*/
 /*  E)  The beginning, end and anchor of the selection. The order is in	*/
-/*	the reding direction. NOT in the order that the selection was	*/
+/*	the reading direction. NOT in the order that the selection was	*/
 /*	made. USE EXTREME CARE WHEN YOU REFER TO THE OLD POSITION IN	*/
 /*	EDITING CODE.							*/
 /*									*/
@@ -51,8 +36,7 @@ typedef struct LayoutPosition
 typedef struct DocumentPosition
     {
     struct BufferItem *	dpBi;
-    int			dpParticule;
-    int			dpStroff;
+    unsigned int	dpStroff;
     } DocumentPosition;
 
 typedef struct SelectionScope
@@ -79,6 +63,8 @@ typedef struct DocumentSelection
 typedef struct PositionGeometry
     {
     int			pgLine;
+    int			pgAtLineHead;
+    int			pgAtLineEnd;
 
     int			pgXPixels;
     LayoutPosition	pgTopPosition;
@@ -110,16 +96,64 @@ typedef struct TableRectangle
     int		trCellRowspan;
     } TableRectangle;
 
+#   define	docSamePosition(b,e) \
+		((b)->dpBi == (e)->dpBi && (b)->dpStroff == (e)->dpStroff)
+
+#   define	docPositionsInsideParagraph(b,e) \
+		((b)->dpBi == (e)->dpBi)
 #   define	docPositionsInsideCell(b,e) \
 		((b)->dpBi->biParent == (e)->dpBi->biParent)
 #   define	docPositionsInsideRow(b,e) \
 		((b)->dpBi->biParent->biParent == (e)->dpBi->biParent->biParent)
 
+#   define	docSelectionInsideParagraph(ds) \
+		docPositionsInsideParagraph(&(ds)->dsBegin,&(ds)->dsEnd)
 #   define	docSelectionInsideCell(ds) \
 		docPositionsInsideCell(&(ds)->dsBegin,&(ds)->dsEnd)
 #   define	docSelectionInsideRow(ds) \
 		docPositionsInsideRow(&(ds)->dsBegin,&(ds)->dsEnd)
 
+/************************************************************************/
+/*									*/
+/*  Describe a selection and its relevance for application tools.	*/
+/*									*/
+/************************************************************************/
+
+typedef struct SelectionDescription
+    {
+    unsigned int	sdDocumentId;
+    int			sdIsSet;
+    int			sdDocumentReadonly;
+    int			sdIsIBarSelection;
+    int			sdIsSingleParagraph;
+    int			sdIsSingleCell;
+    int			sdBeginInTable;
+    int			sdBeginInTableHeader;
+    int			sdIsColSlice;
+    int			sdIsRowSlice;
+    int			sdIsTableSlice;
+    int			sdIsTableRectangle;
+    int			sdIsObjectSelection;
+
+    int			sdCanReplace;
+    int			sdInExternalItem;
+    int			sdInDocumentBody;
+    int			sdInHeaderFooter;
+
+    int			sdBeginInField;
+    int			sdBeginInHyperlink;
+    int			sdBeginInBookmark;
+
+    int			sdIsListBullet;
+
+    int			sdHasLists;
+    int			sdFirstListParaNr;
+    int			sdListOverride;
+    int			sdListLevel;
+    int			sdMultiList;
+    int			sdMultiLevel;
+    } SelectionDescription;
+
 /************************************************************************/
 /*									*/
 /*  Routine Declarations.						*/
@@ -137,9 +171,11 @@ extern void docSetIBarSelection(	DocumentSelection *		ds,
 extern int docIsIBarSelection( const DocumentSelection *		ds );
 extern int docIsParaSelection( const DocumentSelection *		ds );
 
-extern int docGetObjectSelection(	DocumentSelection *	ds,
-					DocumentPosition *	dpObject,
-					InsertedObject **	pIo );
+extern int docGetObjectSelection(
+				const DocumentSelection *	ds,
+				int *				pPart,
+				DocumentPosition *		dpObject,
+				InsertedObject **		pIo );
 
 extern void docSetRangeSelection(
 				DocumentSelection *		ds,
@@ -149,8 +185,10 @@ extern void docSetRangeSelection(
 				int				col0,
 				int				col1 );
 
-extern int docBeginOfLine(	DocumentPosition *		dp );
-extern int docEndOfLine(	DocumentPosition *		dp );
+extern int docBeginOfLine(	DocumentPosition *		dp,
+				int				wasAtHead );
+extern int docEndOfLine(	DocumentPosition *		dp,
+				int				wasAtHead );
 
 extern int docGetTableRectangle(	TableRectangle *		tr,
 					const DocumentSelection *	ds );
@@ -161,4 +199,13 @@ extern int docGetTableSliceSelection(
 				TableRectangle *		tr,
 				const DocumentSelection *	ds );
 
+extern void docDescribeSelection(
+			    SelectionDescription *		sd,
+			    const DocumentSelection *		ds,
+			    const struct BufferDocument *	bd,
+			    unsigned int			documentId,
+			    int					documentRo );
+
+extern void docInitSelectionDescription(	SelectionDescription *	sd );
+
 #   endif	/*	DOC_SELECT_H	*/
