@@ -4,17 +4,18 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<limits.h>
-#   include	<appDebugon.h>
 
 #   include	<utilPropMask.h>
 #   include	<appFrame.h>
 #   include	"appMarginTool.h"
 
+#   include	<appDebugon.h>
+
 /************************************************************************/
 /*									*/
 /*  Show the size of the paper in the Text.				*/
@@ -48,24 +49,27 @@ void appMarginToolShowMargins(	AppMarginTool *			amt,
 /*									*/
 /************************************************************************/
 
-int appMarginToolGetMargins(	unsigned int *		pUpdMask,
-				unsigned int		chgMask,
+int appMarginToolGetMargins(	PropertyMask *		pUpdMask,
+				const PropertyMask *	chgMask,
 				int			unitType,
 				AppMarginTool *		amt,
 				DocumentGeometry *	dg )
     {
     int			changed;
-    unsigned int	updMask= 0;
 
     const int		minValue= 1;
     const int		adaptToMin= 0;
     const int		maxValue= INT_MAX;
     const int		adaptToMax= 0;
 
+    PropertyMask	updMask;
+
+    PROPmaskCLEAR( &updMask );
+
     if  ( PROPmaskISSET( chgMask, DGpropLEFT_MARGIN ) )
 	{
 	if  ( appGetLengthFromTextWidget( amt->amtLeftMarginText,
-				&dg->dgLeftMarginTwips, &changed, unitType,
+				&(dg->dgLeftMarginTwips), &changed, unitType,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	    { return -1;	}
 	if  ( changed )
@@ -75,7 +79,7 @@ int appMarginToolGetMargins(	unsigned int *		pUpdMask,
     if  ( PROPmaskISSET( chgMask, DGpropTOP_MARGIN ) )
 	{
 	if  ( appGetLengthFromTextWidget( amt->amtTopMarginText,
-				&dg->dgTopMarginTwips, &changed, unitType,
+				&(dg->dgTopMarginTwips), &changed, unitType,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	    { return -1;	}
 	if  ( changed )
@@ -85,7 +89,7 @@ int appMarginToolGetMargins(	unsigned int *		pUpdMask,
     if  ( PROPmaskISSET( chgMask, DGpropRIGHT_MARGIN ) )
 	{
 	if  ( appGetLengthFromTextWidget( amt->amtRightMarginText,
-				&dg->dgRightMarginTwips, &changed, unitType,
+				&(dg->dgRightMarginTwips), &changed, unitType,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	    { return -1;	}
 	if  ( changed )
@@ -95,7 +99,7 @@ int appMarginToolGetMargins(	unsigned int *		pUpdMask,
     if  ( PROPmaskISSET( chgMask, DGpropBOTTOM_MARGIN ) )
 	{
 	if  ( appGetLengthFromTextWidget( amt->amtBottomMarginText,
-				&dg->dgBottomMarginTwips, &changed, unitType,
+				&(dg->dgBottomMarginTwips), &changed, unitType,
 				minValue, adaptToMin, maxValue, adaptToMax ) )
 	    { return -1;	}
 	if  ( changed )
@@ -113,11 +117,11 @@ int appMarginToolGetMargins(	unsigned int *		pUpdMask,
 
 int appMarginToolCheckMargins(	const DocumentGeometry * dg )
     {
-    if  ( dg->dgLeftMarginTwips+ dg->dgRightMarginTwips >= 
+    if  ( 1.2* ( dg->dgLeftMarginTwips+ dg->dgRightMarginTwips ) >= 
 						    dg->dgPageWideTwips )
 	{ return -1;	}
 
-    if  ( dg->dgTopMarginTwips+ dg->dgBottomMarginTwips >= 
+    if  ( 1.2* ( dg->dgTopMarginTwips+ dg->dgBottomMarginTwips ) >= 
 						    dg->dgPageWideTwips )
 	{ return -1;	}
 
@@ -130,23 +134,26 @@ int appMarginToolCheckMargins(	const DocumentGeometry * dg )
 /*									*/
 /************************************************************************/
 
-void appMakeMarginToolWidgets(	APP_WIDGET		parent,
-				const MarginToolResources * mtr,
-				AppMarginTool *		amt,
-				APP_TEXT_ACTIVATE_CALLBACK	changed,
-				void *			through )
+void appMakeMarginToolWidgets(	APP_WIDGET			parent,
+				const MarginToolResources *	mtr,
+				AppMarginTool *			amt,
+				APP_TXACTIVATE_CALLBACK_T	gotValue,
+				void *				through )
     {
     APP_WIDGET	label;
     APP_WIDGET	row;
 
-    const int	textWidth= 7;
+    const int	textWidth= 6;
     const int	textEnabled= 1;
+    const int	colspan= 1;
 
     appMakeRowFrameInColumn( &(amt->amtMarginFrame), &(amt->amtInsideWidget),
 					    parent, 2, mtr->mtrTitleText );
 
-    appMakeColumnInRow( &(amt->amtLeftColumn), amt->amtInsideWidget, 0 );
-    appMakeColumnInRow( &(amt->amtRightColumn), amt->amtInsideWidget, 1 );
+    appMakeColumnInRow( &(amt->amtLeftColumn), amt->amtInsideWidget,
+								0, colspan );
+    appMakeColumnInRow( &(amt->amtRightColumn), amt->amtInsideWidget,
+								1, colspan );
 
     /**********************/
 
@@ -164,19 +171,19 @@ void appMakeMarginToolWidgets(	APP_WIDGET		parent,
 				amt->amtRightColumn, mtr->mtrBottomMarginText,
 				textWidth, textEnabled );
 
-    if  ( changed )
+    if  ( gotValue )
 	{
 	appGuiSetGotValueCallbackForText( amt->amtLeftMarginText,
-							changed, through );
+							gotValue, through );
 
 	appGuiSetGotValueCallbackForText( amt->amtTopMarginText,
-							changed, through );
+							gotValue, through );
 
 	appGuiSetGotValueCallbackForText( amt->amtRightMarginText,
-							changed, through );
+							gotValue, through );
 
 	appGuiSetGotValueCallbackForText( amt->amtBottomMarginText,
-							changed, through );
+							gotValue, through );
 	}
 
     return;
