@@ -4,33 +4,38 @@
 /*									*/
 /************************************************************************/
 
-#   include	"config.h"
+#   include	"appFrameConfig.h"
 
 #   include	<stdlib.h>
 #   include	<stdio.h>
 #   include	<stddef.h>
 
+#   include	<appFrame.h>
+
 #   include	<appDebugon.h>
 
-#   include	<appFrame.h>
+typedef struct QuestionContextResources
+    {
+    char *	qcrYesText;
+    char *	qcrNoText;
+    char *	qcrCancelText;
+    char *	qcrOkText;
+    } QuestionContextResources;
 
 typedef struct QuestionContext
     {
-    AppDialog	qcDialog;
-    APP_WIDGET	qcPaned;
+    AppDialog				qcDialog;
+    APP_WIDGET				qcPaned;
 
-    APP_WIDGET	qcQuestionWidget;
-    APP_WIDGET	qcSubjectWidget;
+    APP_WIDGET				qcQuestionWidget;
+    APP_WIDGET				qcSubjectWidget;
 
-    APP_WIDGET	qcYesButton;
-    APP_WIDGET	qcNoButton;
-    APP_WIDGET	qcCancelButton;
-    APP_WIDGET	qcOkButton;
+    APP_WIDGET				qcYesButton;
+    APP_WIDGET				qcNoButton;
+    APP_WIDGET				qcCancelButton;
+    APP_WIDGET				qcOkButton;
 
-    char *	qcYesText;
-    char *	qcNoText;
-    char *	qcCancelText;
-    char *	qcOkText;
+    const QuestionContextResources *	qcResources;
     } QuestionContext;
 
 /************************************************************************/
@@ -40,16 +45,16 @@ typedef struct QuestionContext
 /************************************************************************/
 
 static AppConfigurableResource APP_QuestionResourceTable[]=
-    {
-	APP_RESOURCE( "messageYesText",
-		    offsetof(QuestionContext,qcYesText),	"Yes" ),
-	APP_RESOURCE( "messageNoText",
-		    offsetof(QuestionContext,qcNoText),		"No" ),
-	APP_RESOURCE( "messageCancelText",
-		    offsetof(QuestionContext,qcCancelText),	"Cancel" ),
-	APP_RESOURCE( "messageOkText",
-		    offsetof(QuestionContext,qcOkText),		"Ok" ),
-    };
+{
+    APP_RESOURCE( "messageYesText",
+		offsetof(QuestionContextResources,qcrYesText),	"Yes" ),
+    APP_RESOURCE( "messageNoText",
+		offsetof(QuestionContextResources,qcrNoText),	"No" ),
+    APP_RESOURCE( "messageCancelText",
+		offsetof(QuestionContextResources,qcrCancelText),"Cancel" ),
+    APP_RESOURCE( "messageOkText",
+		offsetof(QuestionContextResources,qcrOkText),	"Ok" ),
+};
 
 /************************************************************************/
 /*									*/
@@ -57,26 +62,35 @@ static AppConfigurableResource APP_QuestionResourceTable[]=
 /*									*/
 /************************************************************************/
 
-static void appQuestionButtonPushed(	APP_WIDGET	button,
-					void *		voidqc,
-					void *		call_data )
+static APP_BUTTON_CALLBACK_H( appQuestionOkButtonPushed, button, voidqc )
     {
     QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    if  ( button == qc->qcOkButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespOK ); return;	}
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespOK );
+    return;
+    }
 
-    if  ( button == qc->qcYesButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespYES ); return;	}
+static APP_BUTTON_CALLBACK_H( appQuestionYesButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    if  ( button == qc->qcNoButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespNO ); return;	}
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespYES );
+    return;
+    }
 
-    if  ( button == qc->qcCancelButton )
-	{ appGuiBreakDialog( &(qc->qcDialog), AQDrespCANCEL ); return;	}
+static APP_BUTTON_CALLBACK_H( appQuestionNoButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
 
-    LDEB(1);
-    appGuiBreakDialog( &(qc->qcDialog), AQDrespFAILURE );
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespNO );
+    return;
+    }
+
+static APP_BUTTON_CALLBACK_H( appQuestionCancelButtonPushed, button, voidqc )
+    {
+    QuestionContext *		qc= (QuestionContext *)voidqc;
+
+    appGuiBreakDialog( &(qc->qcDialog), AQDrespCANCEL );
     return;
     }
 
@@ -98,7 +112,7 @@ static APP_WIDGET appQuestionMakeOkRow(	APP_WIDGET		parent,
     buttonRow= appMakeRowInColumn( parent, 1, heightResizable );
 
     appMakeButtonInRow( &(qc->qcOkButton), buttonRow, okText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+		    appQuestionOkButtonPushed, (void *)qc, 0, showAsDefault );
 
     return buttonRow;
     }
@@ -117,13 +131,16 @@ static APP_WIDGET appQuestionMakeYesNoCancelRow( APP_WIDGET	parent,
     buttonRow= appMakeRowInColumn( parent, 3, heightResizable );
 
     appMakeButtonInRow( &(qc->qcYesButton), buttonRow, yesText,
-		appQuestionButtonPushed, (void *)qc, 0, showYesAsDefault );
+				appQuestionYesButtonPushed,
+				(void *)qc, 0, showYesAsDefault );
 
     appMakeButtonInRow( &(qc->qcNoButton), buttonRow, noText,
-		appQuestionButtonPushed, (void *)qc, 1, showOtherAsDefault );
+				appQuestionNoButtonPushed,
+				(void *)qc, 1, showOtherAsDefault );
 
     appMakeButtonInRow( &(qc->qcCancelButton), buttonRow, cancelText,
-		appQuestionButtonPushed, (void *)qc, 2, showOtherAsDefault );
+				appQuestionCancelButtonPushed,
+				(void *)qc, 2, showOtherAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcYesButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcCancelButton );
@@ -144,10 +161,12 @@ static APP_WIDGET appQuestionMakeOkCancelRow( APP_WIDGET	parent,
     buttonRow= appMakeRowInColumn( parent, 2, heightResizable );
 
     appMakeButtonInRow( &(qc->qcOkButton), buttonRow, okText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+				appQuestionOkButtonPushed,
+				(void *)qc, 0, showAsDefault );
 
     appMakeButtonInRow( &(qc->qcCancelButton), buttonRow, cancelText,
-		    appQuestionButtonPushed, (void *)qc, 1, showAsDefault );
+				appQuestionCancelButtonPushed,
+				(void *)qc, 1, showAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcOkButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcCancelButton );
@@ -169,10 +188,12 @@ static APP_WIDGET appQuestionMakeYesNoRow( APP_WIDGET		parent,
     buttonRow= appMakeRowInColumn( parent, 2, heightResizable );
 
     appMakeButtonInRow( &(qc->qcYesButton), buttonRow, yesText,
-		    appQuestionButtonPushed, (void *)qc, 0, showAsDefault );
+					    appQuestionYesButtonPushed,
+					    (void *)qc, 0, showAsDefault );
 
     appMakeButtonInRow( &(qc->qcNoButton), buttonRow, noText,
-		    appQuestionButtonPushed, (void *)qc, 1, showAsDefault );
+					    appQuestionNoButtonPushed,
+					    (void *)qc, 1, showAsDefault );
 
     appGuiSetDefaultButtonForDialog( &(qc->qcDialog), qc->qcYesButton );
     appGuiSetCancelButtonForDialog( &(qc->qcDialog), qc->qcNoButton );
@@ -181,10 +202,12 @@ static APP_WIDGET appQuestionMakeYesNoRow( APP_WIDGET		parent,
     }
 
 /************************************************************************/
+/*									*/
 /*  Make the framework of a Question Dialog.				*/
+/*									*/
 /************************************************************************/
 
-static APP_DESTROY_CALLBACK( appFreeQuestionContext, w, voidqc )
+static APP_DESTROY_CALLBACK_H( appFreeQuestionContext, w, voidqc )
     { free( voidqc ); return;	}
 
 static int appMakeQuestionDialog( 	EditApplication *	ea,
@@ -195,6 +218,9 @@ static int appMakeQuestionDialog( 	EditApplication *	ea,
 
     QuestionContext *	qc;
 
+    static int				gotResources= 0;
+    static QuestionContextResources	qcr;
+
     qc= (QuestionContext *)malloc( sizeof(QuestionContext) );
     if  ( ! qc )
 	{ XDEB(qc); return -1;	}
@@ -205,14 +231,18 @@ static int appMakeQuestionDialog( 	EditApplication *	ea,
     qc->qcNoButton= (APP_WIDGET)0;
     qc->qcCancelButton= (APP_WIDGET)0;
     qc->qcOkButton= (APP_WIDGET)0;
+    qc->qcResources= &qcr;
 
-    appGuiGetResourceValues( ea, (void *)qc,
+    if  ( ! gotResources )
+	{
+	appGuiGetResourceValues( &gotResources, ea, (void *)&qcr,
 				    APP_QuestionResourceTable,
 				    sizeof(APP_QuestionResourceTable)/
 				    sizeof(AppConfigurableResource) );
+	}
 
     appMakeVerticalDialog( &(qc->qcDialog), &paned, ea,
-						(APP_CALLBACK_FUNC)0,
+						(APP_CLOSE_CALLBACK_T)0,
 						appFreeQuestionContext,
 						(void *)qc,
 						ea->eaMessageDialogName );
@@ -334,9 +364,11 @@ static int appQuestionRunAnyYesNoCancelDialog(	EditApplication *	ea,
 						const char *	noText,
 						const char *	cancelText )
     {
-    appQuestionChangeYesText( qc, yesText?yesText:qc->qcYesText );
-    appQuestionChangeNoText( qc, noText?noText:qc->qcNoText );
-    appQuestionChangeCancelText( qc, cancelText?cancelText:qc->qcCancelText );
+    const QuestionContextResources *	qcr= qc->qcResources;
+
+    appQuestionChangeYesText( qc, yesText?yesText:qcr->qcrYesText );
+    appQuestionChangeNoText( qc, noText?noText:qcr->qcrNoText );
+    appQuestionChangeCancelText( qc, cancelText?cancelText:qcr->qcrCancelText );
 
     appQuestionRunDialog( ea, qc, relative, option );
 
@@ -350,8 +382,10 @@ static int appQuestionRunAnyOkCancelDialog(	EditApplication *	ea,
 						const char *	okText,
 						const char *	cancelText )
     {
-    appQuestionChangeOkText( qc, okText?okText:qc->qcOkText );
-    appQuestionChangeCancelText( qc, cancelText?cancelText:qc->qcCancelText );
+    const QuestionContextResources *	qcr= qc->qcResources;
+
+    appQuestionChangeOkText( qc, okText?okText:qcr->qcrOkText );
+    appQuestionChangeCancelText( qc, cancelText?cancelText:qcr->qcrCancelText );
 
     appQuestionRunDialog( ea, qc, relative, option );
 
@@ -365,8 +399,10 @@ static int appQuestionRunAnyYesNoDialog(	EditApplication *	ea,
 						const char *	yesText,
 						const char *	noText )
     {
-    appQuestionChangeYesText( qc, yesText?yesText:qc->qcYesText );
-    appQuestionChangeNoText( qc, noText?noText:qc->qcNoText );
+    const QuestionContextResources *	qcr= qc->qcResources;
+
+    appQuestionChangeYesText( qc, yesText?yesText:qcr->qcrYesText );
+    appQuestionChangeNoText( qc, noText?noText:qcr->qcrNoText );
 
     appQuestionRunDialog( ea, qc, relative, option );
 
@@ -386,15 +422,24 @@ int appQuestionRunSubjectYesNoCancelDialog(	EditApplication *	ea,
 
     static QuestionContext *	subjectYesNoCancelContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: \"%s\" %s\n", ea->eaApplicationName, subject, question );
+	return AQDrespCANCEL;
+	}
+
     if  ( ! subjectYesNoCancelContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &subjectYesNoCancelContext, &paned ) )
 	    { LDEB(1); return AQDrespFAILURE;	}
 
+	qcr= subjectYesNoCancelContext->qcResources;
+
 	appQuestionMakeSubjectQuestion( subjectYesNoCancelContext, paned,
 							subject, question );
 
@@ -402,9 +447,9 @@ int appQuestionRunSubjectYesNoCancelDialog(	EditApplication *	ea,
 
 	buttonRow= appQuestionMakeYesNoCancelRow( paned,
 				    subjectYesNoCancelContext,
-				    subjectYesNoCancelContext->qcYesText,
-				    subjectYesNoCancelContext->qcNoText,
-				    subjectYesNoCancelContext->qcCancelText );
+				    qcr->qcrYesText,
+				    qcr->qcrNoText,
+				    qcr->qcrCancelText );
 	}
     else{
 	appQuestionChangeSubject( subjectYesNoCancelContext, subject );
@@ -433,24 +478,33 @@ int appQuestionRunYesNoCancelDialog(	EditApplication *	ea,
 
     static QuestionContext *	yesNoCancelContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: %s\n", ea->eaApplicationName, question );
+	return AQDrespCANCEL;
+	}
+
     if  ( ! yesNoCancelContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &yesNoCancelContext, &paned ) )
 	    { LDEB(1); return AQDrespFAILURE;	}
 
+	qcr= yesNoCancelContext->qcResources;
+
 	appQuestionMakeQuestion( yesNoCancelContext, paned, question );
 
 	appGuiInsertSeparatorInColumn( &sep, paned );
 
 	buttonRow= appQuestionMakeYesNoCancelRow( paned,
 					yesNoCancelContext,
-					yesNoCancelContext->qcYesText,
-					yesNoCancelContext->qcNoText,
-					yesNoCancelContext->qcCancelText );
+					qcr->qcrYesText,
+					qcr->qcrNoText,
+					qcr->qcrCancelText );
 	}
     else{
 	appQuestionChangeQuestion( yesNoCancelContext, question );
@@ -474,28 +528,39 @@ void appQuestionRunSubjectErrorDialog(	EditApplication *	ea,
     {
     static QuestionContext *	subjectErrorContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: \"%s\" %s\n", ea->eaApplicationName, subject, message );
+	return;
+	}
+
     if  ( ! subjectErrorContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &subjectErrorContext, &paned ) )
 	    { LDEB(1); return;	}
 
+	qcr= subjectErrorContext->qcResources;
+
 	appQuestionMakeSubjectQuestion( subjectErrorContext, paned,
 							subject, message );
 
 	appGuiInsertSeparatorInColumn( &sep, paned );
 
 	buttonRow= appQuestionMakeOkRow( paned, subjectErrorContext,
-					    subjectErrorContext->qcOkText );
+							qcr->qcrOkText );
 	}
     else{
+	const QuestionContextResources *	qcr= subjectErrorContext->qcResources;
+
 	appQuestionChangeSubject( subjectErrorContext, subject );
 	appQuestionChangeQuestion( subjectErrorContext, message );
 	appQuestionChangeOkText( subjectErrorContext,
-					    subjectErrorContext->qcOkText );
+					    qcr->qcrOkText );
 	}
 
     appQuestionRunOkDialog( ea, relative, option, subjectErrorContext );
@@ -515,23 +580,32 @@ int appQuestionRunOkCancelDialog(	EditApplication *	ea,
 
     static QuestionContext *	okCancelContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: %s\n", ea->eaApplicationName, question );
+	return AQDrespCANCEL;
+	}
+
     if  ( ! okCancelContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &okCancelContext, &paned ) )
 	    { LDEB(1); return AQDrespFAILURE;	}
 
+	qcr= okCancelContext->qcResources;
+
 	appQuestionMakeQuestion( okCancelContext, paned, question );
 
 	appGuiInsertSeparatorInColumn( &sep, paned );
 
 	buttonRow= appQuestionMakeOkCancelRow( paned,
 					okCancelContext,
-					okCancelContext->qcOkText,
-					okCancelContext->qcCancelText );
+					qcr->qcrOkText,
+					qcr->qcrCancelText );
 	}
     else{
 	appQuestionChangeQuestion( okCancelContext, question );
@@ -558,15 +632,24 @@ int appQuestionRunSubjectOkCancelDialog( EditApplication *	ea,
 
     static QuestionContext *	subjectOkCancelContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: \"%s\" %s\n", ea->eaApplicationName, subject, message );
+	return AQDrespCANCEL;
+	}
+
     if  ( ! subjectOkCancelContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &subjectOkCancelContext, &paned ) )
 	    { LDEB(1); return AQDrespFAILURE;	}
 
+	qcr= subjectOkCancelContext->qcResources;
+
 	appQuestionMakeSubjectQuestion( subjectOkCancelContext, paned,
 							subject, message );
 
@@ -574,8 +657,8 @@ int appQuestionRunSubjectOkCancelDialog( EditApplication *	ea,
 
 	buttonRow= appQuestionMakeOkCancelRow( paned,
 					subjectOkCancelContext,
-					subjectOkCancelContext->qcOkText,
-					subjectOkCancelContext->qcCancelText );
+					qcr->qcrOkText,
+					qcr->qcrCancelText );
 	}
     else{
 	appQuestionChangeQuestion( subjectOkCancelContext, message );
@@ -603,15 +686,24 @@ int appQuestionRunSubjectYesNoDialog(	EditApplication *	ea,
 
     static QuestionContext *	subjectYesNoContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: \"%s\" %s\n", ea->eaApplicationName, subject, message );
+	return AQDrespFAILURE;
+	}
+
     if  ( ! subjectYesNoContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &subjectYesNoContext, &paned ) )
 	    { LDEB(1); return AQDrespFAILURE;	}
 
+	qcr= subjectYesNoContext->qcResources;
+
 	appQuestionMakeSubjectQuestion( subjectYesNoContext, paned,
 							subject, message );
 
@@ -619,8 +711,8 @@ int appQuestionRunSubjectYesNoDialog(	EditApplication *	ea,
 
 	buttonRow= appQuestionMakeYesNoRow( paned,
 					subjectYesNoContext,
-					subjectYesNoContext->qcYesText,
-					subjectYesNoContext->qcNoText );
+					qcr->qcrYesText,
+					qcr->qcrNoText );
 	}
     else{
 	appQuestionChangeQuestion( subjectYesNoContext, message );
@@ -643,25 +735,36 @@ void appQuestionRunErrorDialog(	EditApplication *	ea,
     {
     static QuestionContext *	errorContext;
 
+    if  ( ! ea->eaToplevel.atTopWidget )
+	{
+	appDebug( "%s: %s\n", ea->eaApplicationName, message );
+	return;
+	}
+
     if  ( ! errorContext )
 	{
-	APP_WIDGET			paned;
-	APP_WIDGET			sep;
-	APP_WIDGET			buttonRow;
+	APP_WIDGET				paned;
+	APP_WIDGET				sep;
+	APP_WIDGET				buttonRow;
+	const QuestionContextResources *	qcr;
 
 	if  ( appMakeQuestionDialog( ea, &errorContext, &paned ) )
 	    { LDEB(1); return;	}
 
+	qcr= errorContext->qcResources;
+
 	appQuestionMakeQuestion( errorContext, paned, message );
 
 	appGuiInsertSeparatorInColumn( &sep, paned );
 
 	buttonRow= appQuestionMakeOkRow( paned,
-				    errorContext, errorContext->qcOkText );
+				    errorContext, qcr->qcrOkText );
 	}
     else{
+	const QuestionContextResources *	qcr= errorContext->qcResources;
+
 	appQuestionChangeQuestion( errorContext, message );
-	appQuestionChangeOkText( errorContext, errorContext->qcOkText );
+	appQuestionChangeOkText( errorContext, qcr->qcrOkText );
 	}
 
     appQuestionRunOkDialog( ea, relative, option, errorContext );
