@@ -13,7 +13,13 @@
 #   include	<time.h>
 
 #   include	<utilPs.h>
+#   include	<utilTree.h>
 #   include	<geo2DInteger.h>
+#   include	<utilFontmap.h>
+#   include	<utilPostscriptFace.h>
+#   include	<appSystem.h>
+#   include	<sioStdio.h>
+#   include	<sioPfb.h>
 
 #   include	<appDebugon.h>
 
@@ -49,34 +55,141 @@ static void utilPsBoundingBoxComment(
 /*									*/
 /************************************************************************/
 
-static void appPsListFontNames(	SimpleOutputStream *		sos,
-				const PostScriptFaceList *	psfl )
+static void utilPsGetEmbed(		PostScriptFace *	psf,
+					const char *		fontDirectory )
     {
-    if  ( psfl->psflFaceCount > 0 )
+    const char *	fontFile;
+
+    fontFile= utilFontmapGetEntry( psf->psfAfi->afiFontName );
+    if  ( fontFile )
 	{
-	int			i;
-	const PostScriptFace *	psf= psfl->psflFaces;
+	int		relativeIsFile= 0;
+	char		absolute[1000+1];
+	int		l;
 
-	sioOutPrintf( sos, "%%%%DocumentFonts: %s\n",
-						psf->psfAfi->afiFontName );
-	psf++;
-
-	for ( i= 1; i < psfl->psflFaceCount; psf++, i++ )
+	l= appAbsoluteName( absolute, sizeof(absolute)-1, fontFile,
+					    relativeIsFile, fontDirectory );
+	if  ( l >= 0 )
 	    {
-	    int		done;
-
-	    for ( done= 0; done < i; done++ )
+	    if  ( ! appTestFileExists( absolute ) )
 		{
-		if  ( ! strcmp( psfl->psflFaces[done].psfAfi->afiFontName,
-						psf->psfAfi->afiFontName ) )
-		    { break;	}
+		const char *	extension;
+
+		if  ( psf->psfFontFileName )
+		    { free( psf->psfFontFileName );	}
+
+		psf->psfFontFileName= strdup( absolute );
+		if  ( ! psf->psfFontFileName )
+		    { XDEB(psf->psfFontFileName); return;	}
+
+		extension= appFileExtensionOfName( psf->psfFontFileName );
+
+		psf->psfEmbed= PSembedNO;
+
+		if  ( extension &&
+		      ( ! strcmp( extension, "pfb" ) 	||
+			! strcmp( extension, "PFB" ) 	)	)
+		    { psf->psfEmbed= PSembedBTOA;	}
+
+		if  ( extension &&
+		      ( ! strcmp( extension, "pfa" ) 	||
+			! strcmp( extension, "PFA" ) 	)	)
+		    { psf->psfEmbed= PSembedCOPY;	}
+
+		if  ( extension &&
+		      ( ! strcmp( extension, "ttf" ) 	||
+			! strcmp( extension, "TTF" ) 	)	)
+		    { psf->psfEmbed= PSembedTTFTO42;	}
+
+		psf->psfFontFileNameLength= l;
+		}
+	    else{ psf->psfEmbed= PSembedNO;	}
+	    }
+	}
+    else{ psf->psfEmbed= PSembedNO;	}
+
+    return;
+    }
+
+static void utilPsListFontNames(	SimpleOutputStream *		sos,
+					const PostScriptTypeList *	pstl )
+    {
+    PostScriptFace *	psf;
+    const char *	fontName;
+    int			done;
+
+    if  ( ! pstl->pstlFaceTree )
+	{ return;	}
+
+    done= 0;
+    psf= (PostScriptFace *)utilTreeGetFirst( pstl->pstlFaceTree,
+								&fontName );
+    while( psf )
+	{
+	if  ( done > 0 )
+	    {
+	    sioOutPrintf( sos, "%%%%+ %s\n", psf->psfAfi->afiFontName );
+	    }
+	else{
+	    sioOutPrintf( sos, "%%%%DocumentFonts: %s\n",
+					    psf->psfAfi->afiFontName );
+	    }
+
+	done++;
+	psf= (PostScriptFace *)utilTreeGetNext( pstl->pstlFaceTree,
+							    &fontName );
+	}
+
+    if  ( pstl->pstlFontDirectory )
+	{
+	done= 0;
+	psf= (PostScriptFace *)utilTreeGetFirst( pstl->pstlFaceTree,
+								&fontName );
+	while( psf )
+	    {
+	    if  ( psf->psfEmbed == PSembedUNKNOWN )
+		{ utilPsGetEmbed( psf, pstl->pstlFontDirectory );	}
+
+	    if  ( psf->psfEmbed > 0 )
+		{
+		if  ( done > 0 )
+		    {
+		    sioOutPrintf( sos, "%%%%+ %s\n", psf->psfAfi->afiFontName );
+		    }
+		else{
+		    sioOutPrintf( sos, "%%%%DocumentSuppliedFonts: %s\n",
+						    psf->psfAfi->afiFontName );
+		    }
+
+		done++;
 		}
 
-	    if  ( done >= i )
+	    psf= (PostScriptFace *)utilTreeGetNext( pstl->pstlFaceTree,
+								&fontName );
+	    }
+	}
+
+    done= 0;
+    psf= (PostScriptFace *)utilTreeGetFirst( pstl->pstlFaceTree,
+							    &fontName );
+    while( psf )
+	{
+	if  ( psf->psfEmbed <= 0 )
+	    {
+	    if  ( done > 0 )
 		{
 		sioOutPrintf( sos, "%%%%+ %s\n", psf->psfAfi->afiFontName );
 		}
+	    else{
+		sioOutPrintf( sos, "%%%%DocumentNeededFonts: %s\n",
+						psf->psfAfi->afiFontName );
+		}
+
+	    done++;
 	    }
+
+	psf= (PostScriptFace *)utilTreeGetNext( pstl->pstlFaceTree,
+							    &fontName );
 	}
 
     return;
@@ -146,7 +259,7 @@ static void utilPsFoldComment(	SimpleOutputStream *	sos,
 
 void utilPsStartDSCDocument(
 			const PrintingState *		ps,
-			const PostScriptFaceList *	psfl,
+			const PostScriptTypeList *	pstl,
 			const char *			title,
 			const char *			creatorName,
 			const char *			creatorReference )
@@ -164,8 +277,8 @@ void utilPsStartDSCDocument(
     sioOutPrintf( sos, "%%%%Pages: (atend)\n" );
     sioOutPrintf( sos, "%%%%PageOrder: Ascend\n" );
 
-    if  ( psfl )
-	{ appPsListFontNames( sos, psfl );	}
+    if  ( pstl )
+	{ utilPsListFontNames( sos, pstl );	}
 
     sioOutPrintf( sos, "%%%%EndComments\n" );
 
@@ -240,11 +353,12 @@ void utilPsRefreshNupSchema(
 /*									*/
 /************************************************************************/
 
-void appPsPrintString(	SimpleOutputStream *	sos,
+void utilPsPrintString(	PrintingState *		ps,
 			const unsigned char *	s,
 			int			len )
     {
-    int		i;
+    SimpleOutputStream *	sos= ps->psSos;
+    int				i;
 
     for ( i= 0; i < len; s++, i++ )
 	{
@@ -275,6 +389,36 @@ void appPsPrintString(	SimpleOutputStream *	sos,
     return;
     }
 
+void utilPsPrintStringValue(	PrintingState *		ps,
+				const unsigned char *	s,
+				int			len )
+    {
+    SimpleOutputStream *	sos= ps->psSos;
+
+    sioOutPutCharacter( '(', sos );
+    utilPsPrintString( ps, s, len );
+    sioOutPutCharacter( ')', sos );
+
+    return;
+    }
+
+void utilPsMovePrintStringValue(	PrintingState *		ps,
+					const unsigned char *	s,
+					int			len,
+					int			x,
+					int			y )
+    {
+    SimpleOutputStream *	sos= ps->psSos;
+
+    sioOutPutCharacter( '(', sos );
+    utilPsPrintString( ps, s, len );
+    sioOutPrintf( sos, ") %d %d mvs\n", x, y );
+
+    ps->psLastPageMarked= ps->psPagesPrinted;
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Set the font, this depends on the assumption that fonts have been	*/
@@ -285,11 +429,12 @@ void appPsPrintString(	SimpleOutputStream *	sos,
 /*									*/
 /************************************************************************/
 
-void utilPsSetFont(	SimpleOutputStream *		sos,
+void utilPsSetFont(	PrintingState *			ps,
 			const char *			prefix,
 			const TextAttribute *		ta )
     {
-    int		fontSizeTwips= 10* ta->taFontSizeHalfPoints;
+    SimpleOutputStream *	sos= ps->psSos;
+    int				fontSizeTwips= 10* ta->taFontSizeHalfPoints;
 
     if  ( ta->taSuperSub == DOCfontSUPERSCRIPT	||
 	  ta->taSuperSub == DOCfontSUBSCRIPT	)
@@ -310,6 +455,8 @@ void utilPsSetFont(	SimpleOutputStream *		sos,
 	{ sioOutPutCharacter( 'i', sos );	}
     sioOutPutCharacter( '\n', sos );
 
+    ps->psLastPageMarked= ps->psPagesPrinted;
+
     return;
     }
 
@@ -321,11 +468,13 @@ void utilPsSetFont(	SimpleOutputStream *		sos,
 
 void appPsInitPrintingState(	PrintingState *	ps )
     {
-    int			i;
-
     ps->psSos= (SimpleOutputStream *)0;
+
+    ps->psLastPageMarked= -1;
+    ps->psLastSheetMarked= -1;
     ps->psPagesPrinted= 0;
     ps->psSheetsPrinted= 0;
+    ps->psSheetsStarted= 0;
 
     utilInitPrintGeometry( &(ps->psPrintGeometry) );
 
@@ -349,8 +498,7 @@ void appPsInitPrintingState(	PrintingState *	ps )
     ps->psUsePostScriptFilters= 1;
     ps->psUsePostScriptIndexedImages= 1;
 
-    for ( i= 0; i < ENCODINGps_COUNT; i++ )
-	{ ps->psEncodingDefined[i]= 0;	}
+    utilCleanPrintGeometry( &(ps->psPrintGeometry) );
 
     return;
     }
@@ -377,6 +525,8 @@ void utilPsSetRgbColor(	PrintingState *		ps,
 	{ sioOutPrintf( ps->psSos, "%g setgray\n", r );			}
     else{ sioOutPrintf( ps->psSos, "%g %g %g setrgbcolor\n", r, g, b );	}
 
+    ps->psLastPageMarked= ps->psPagesPrinted;
+
     return;
     }
 
@@ -386,11 +536,12 @@ void utilPsSetRgbColor(	PrintingState *		ps,
 /*									*/
 /************************************************************************/
 
-static void appPsPageOperator(		const char *		operator,
+static void utilPsPageOperator(		const char *		operator,
 					const PrintingState *	ps,
 					int			documentPage )
     {
-    sioOutPrintf( ps->psSos, "%s %% Page %d # %d Sheet %d\n", operator,
+    sioOutPrintf( ps->psSos, "%s %% Page doc:%d -> print:%d Sheet %d\n",
+				    operator,
 				    documentPage+ 1,
 				    ps->psPagesPrinted+ 1,
 				    ps->psSheetsPrinted+ 1 );
@@ -399,7 +550,6 @@ static void appPsPageOperator(		const char *		operator,
 void utilPsStartPage(	PrintingState *			ps,
 			int				documentPage )
     {
-    const AffineTransform2D *	at;
     int				nup= ps->psNupSchema.nsNup;
     int				firstOnSheet= 0;
 
@@ -407,6 +557,9 @@ void utilPsStartPage(	PrintingState *			ps,
 	  ps->psPagesPrinted % nup == 0	)
 	{ firstOnSheet= 1;	}
 
+    if  ( ps->psSheetsStarted > ps->psSheetsPrinted )
+	{ firstOnSheet= 0;	}
+
     if  ( firstOnSheet )
 	{
 	if  ( nup == 1 )
@@ -422,12 +575,13 @@ void utilPsStartPage(	PrintingState *			ps,
 	utilPsBoundingBoxComment( ps, "PageBoundingBox", "PageOrientation" );
 
 	sioOutPrintf( ps->psSos, "%%%%BeginPageSetup\n" );
+	ps->psSheetsStarted= ps->psSheetsPrinted+ 1;
 	}
 
     utilNupGetPageTranform( &(ps->psCurrentTransform),
 				    &(ps->psNupSchema), ps->psPagesPrinted );
 
-    appPsPageOperator( "gsave", ps, documentPage );
+    utilPsPageOperator( "gsave", ps, documentPage );
 
 #   if 0
     if  ( firstOnSheet )
@@ -445,14 +599,8 @@ void utilPsStartPage(	PrintingState *			ps,
 	}
 #   endif
 
-    at= &(ps->psCurrentTransform);
-    sioOutPrintf( ps->psSos, "[ %g %g %g %g %g %g ] concat\n",
-							at->at2Axx,
-							at->at2Axy,
-							at->at2Ayx,
-							at->at2Ayy,
-							at->at2Tx,
-							at->at2Ty );
+    utilPsTransformMatrix( ps->psSos, &(ps->psCurrentTransform) );
+    sioOutPrintf( ps->psSos, " concat\n" );
 
     if  ( firstOnSheet )
 	{ sioOutPrintf( ps->psSos, "%%%%EndPageSetup\n" );	}
@@ -460,17 +608,60 @@ void utilPsStartPage(	PrintingState *			ps,
     return;
     }
 
+void utilPsTransformMatrix(	SimpleOutputStream *		sos,
+				const AffineTransform2D *	at )
+    {
+    sioOutPrintf( sos, "[ %g %g %g %g %g %g ]",
+				    at->at2Axx, at->at2Axy,
+				    at->at2Ayx, at->at2Ayy,
+				    at->at2Tx, at->at2Ty );
+    return;
+    }
+
+/************************************************************************/
+/*									*/
+/*  Finish the page.							*/
+/*									*/
+/*  1)  Print something invisible on absolutely empty pages: Some	*/
+/*	versions of acroread (5 for windows) do not like the pdf that	*/
+/*	ghostscript generates for an absolutely empty page.		*/
+/*									*/
+/************************************************************************/
+
+/*  1  */
+static void utilPsDrawSometingInvisible(	PrintingState *	ps )
+    {
+    sioOutPrintf( ps->psSos,
+	    "1 setgray 0 0 moveto 1 0 rlineto stroke"
+	    "  %% Avoid an empty page\n" );
+    }
+
 void utilPsFinishPage(	PrintingState *		ps,
 			int			documentPage,
 			int			asLast )
     {
-    int				nup= ps->psNupSchema.nsNup;
+    int			nup= ps->psNupSchema.nsNup;
+    int			pageIsEmpty;
+    int			sheetIsEmpty;
+
+    pageIsEmpty= ps->psLastPageMarked < ps->psPagesPrinted;
+    sheetIsEmpty= ps->psLastSheetMarked < ps->psSheetsPrinted;
+
+    if  ( ! pageIsEmpty )
+	{
+	ps->psLastSheetMarked= ps->psSheetsPrinted;
+	sheetIsEmpty= 0;
+	}
 
     if  ( asLast				||
 	  nup == 1				||
 	  ( ps->psPagesPrinted+ 1 ) % nup == 0	)
 	{
-	appPsPageOperator( "showpage grestore", ps, documentPage );
+	/*  1  */
+	if  ( sheetIsEmpty )
+	    { utilPsDrawSometingInvisible( ps );	}
+
+	utilPsPageOperator( "showpage grestore", ps, documentPage );
 
 	sioOutPrintf( ps->psSos, "%%%%PageTrailer\n" );
 
@@ -478,7 +669,7 @@ void utilPsFinishPage(	PrintingState *		ps,
 	ps->psSheetsPrinted++;
 	}
     else{
-	appPsPageOperator( "grestore", ps, documentPage );
+	utilPsPageOperator( "grestore", ps, documentPage );
 
 	ps->psPagesPrinted++;
 	}
@@ -493,6 +684,32 @@ void utilPsFinishPage(	PrintingState *		ps,
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Abort a page as there is a reason not to print it. Usually, this is	*/
+/*  because it is empty.						*/
+/*									*/
+/************************************************************************/
+
+void utilPsAbortPage(	PrintingState *		ps,
+			int			documentPage,
+			int			asLast )
+    {
+    if  ( asLast					&&
+	  ps->psLastSheetMarked == ps->psSheetsPrinted	)
+	{ utilPsPageOperator( "showpage grestore", ps, documentPage );	}
+    else{ utilPsPageOperator( "grestore", ps, documentPage );		}
+
+    if  ( asLast )
+	{
+	sioOutPrintf( ps->psSos, "%%%%Trailer\n" );
+	sioOutPrintf( ps->psSos, "%%%%Pages: %d\n", ps->psSheetsPrinted );
+	sioOutPrintf( ps->psSos, "%%%%EOF\n" );
+	}
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Define a font encoding.						*/
@@ -503,7 +720,7 @@ void utilPsFinishPage(	PrintingState *		ps,
 /*									*/
 /************************************************************************/
 
-static void appPsDefineFontEncoding(
+static void utilPsDefineFontEncoding(
 				SimpleOutputStream *	sos,
 				const char *		name,
 				const char * const *	glyphNames )
@@ -514,18 +731,17 @@ static void appPsDefineFontEncoding(
     sioOutPrintf( sos, "/%s 256 array def\n", name );
 
     /*  2  */
-    sioOutPrintf( sos, "0 1 255 { %s exch /.notdef put } for\n", name );
+    sioOutPrintf( sos, "0 1 255 { %s exch /.notdef put } for %s\n",
+							    name, name );
 
     /*  3  */
-    sioOutPrintf( sos, "%s\n", name );
-
     for ( i= 0; i < 256; i++ )
 	{
 	if  ( glyphNames[i] )
 	    { sioOutPrintf( sos, "  dup %3d /%s put\n", i, glyphNames[i] ); }
 	}
 
-    sioOutPrintf( sos, "pop\n\n" );
+    sioOutPrintf( sos, "readonly\n\n" );
 
     return;
     }
@@ -534,104 +750,267 @@ static void appPsDefineFontEncoding(
 /*									*/
 /*  Make function names for setting the fonts in a list.		*/
 /*									*/
+/*  1)  Define all standard encodings.					*/
+/*  2)  Define all encoded fonts.					*/
+/*									*/
 /************************************************************************/
 
+static void utilPsDefineEncodedFont(
+				SimpleOutputStream *	sos,
+				const AfmFontInfo *	afi,
+				const char *		encodingSuffix,
+				const char *		encodingArrayName,
+				const char *		exceptionSuffix )
+    {
+    sioOutPrintf( sos, "/%s findfont dup length dict begin\n",
+							afi->afiFontName );
+    sioOutPrintf( sos, "  {\n" );
+    sioOutPrintf( sos, "    1 index /FID ne\n" );
+    sioOutPrintf( sos, "      { def } { pop pop } ifelse\n" );
+    sioOutPrintf( sos, "  } forall\n");
+    sioOutPrintf( sos, "  /Encoding %s%s def currentdict\n",
+					encodingArrayName, exceptionSuffix );
+    sioOutPrintf( sos, "end " );
+
+    sioOutPrintf( sos, "/%s%s%s exch definefont pop\n\n",
+				afi->afiFontName,
+				encodingSuffix?encodingSuffix:"",
+				exceptionSuffix );
+
+    return;
+    }
+
+int appPsIncludeFonts(	PrintingState *			ps,
+			const PostScriptTypeList *	pstl )
+    {
+    const char *		fontName;
+    PostScriptFace *		psf;
+
+    SimpleOutputStream *	sos= ps->psSos;
+
+    psf= (PostScriptFace *)utilTreeGetFirst( pstl->pstlFaceTree,
+								&fontName );
+    while( psf )
+	{
+	if  ( psf->psfEmbed == PSembedUNKNOWN )
+	    { utilPsGetEmbed( psf, pstl->pstlFontDirectory );	}
+
+	if  ( psf->psfEmbed > 0 )
+	    {
+	    SimpleInputStream *	sisFile= (SimpleInputStream *)0;
+	    SimpleInputStream *	sisPfb= (SimpleInputStream *)0;
+	    SimpleInputStream *	sis= (SimpleInputStream *)0;
+
+	    sisFile= sioInStdioOpen( psf->psfFontFileName );
+	    if  ( ! sisFile )
+		{ SXDEB(psf->psfFontFileName,sisFile); return -1;	}
+	    sis= sisFile;
+
+	    sioOutPrintf( sos, "\n" );
+	    sioOutPrintf( sos, "%%%%%%%%\n" );
+	    sioOutPrintf( sos, "%%%%%%%%    %s\n", psf->psfAfi->afiFullName );
+	    sioOutPrintf( sos, "%%%%%%%% :: %s\n", psf->psfFontFileName );
+	    sioOutPrintf( sos, "%%%%%%%%\n" );
+
+	    sioOutPrintf( sos, "%%%%BeginFont %s\n",
+					    psf->psfAfi->afiFontName );
+
+	    switch( psf->psfEmbed )
+		{
+		int		c;
+		int		atBol;
+
+		case PSembedBTOA:
+		    sisPfb= sioInPfbOpen( sisFile );
+		    if  ( ! sisPfb )
+			{
+			SXDEB(psf->psfFontFileName,sisPfb);
+			sioInClose( sisFile );
+			return -1;
+			}
+
+		    sis= sisPfb;
+
+		    /*FALLTHROUGH*/
+		case PSembedCOPY:
+		    atBol= 1;
+		    while( ( c= sioInGetCharacter( sis ) ) != EOF )
+			{
+			sioOutPutCharacter( c, sos );
+			atBol= c == '\n';
+			}
+
+		    if  ( ! atBol )
+			{ sioOutPutCharacter( '\n', sos );	}
+
+		    break;
+
+		case PSembedTTFTO42:
+		    if  ( psTtfToPf42( sos,  psf->psfFontFileName, sisFile ) )
+			{
+			SDEB(psf->psfFontFileName);
+			sioInClose( sisFile );
+			return -1;
+			}
+
+		    break;
+
+		default:
+		    LDEB(psf->psfEmbed);
+		    sioInClose( sisFile );
+		    return -1;
+		}
+
+	    sioOutPrintf( sos, "%%%%EndFont %s\n",
+					    psf->psfAfi->afiFontName );
+
+	    if  ( sisPfb )
+		{ sioInClose( sisPfb );	}
+	    if  ( sisFile )
+		{ sioInClose( sisFile );	}
+	    }
+
+	psf= (PostScriptFace *)utilTreeGetNext( pstl->pstlFaceTree,
+								&fontName );
+	}
+
+    return 0;
+    }
+
 void appPsFontNames(	PrintingState *			ps,
-			const PostScriptFaceList *	psfl,
-			int *				encodingDefined,
+			const PostScriptTypeList *	pstl,
 			int				allFonts )
     {
     SimpleOutputStream *	sos= ps->psSos;
 
-    const PostScriptFace *	psf;
+    const char *		fontName;
+    PostScriptFace *		psf;
 
-    int				i;
-    int				started= 0;
+    int				enc;
+    const FontCharset *		fc;
 
-    const char *		encodingSuffix= (const char *)0;
-    const char *		encodingArray= (const char *)0;
+    int				faceNumber= 1;
 
-    psf= psfl->psflFaces;
-    for ( i= 0; i < psfl->psflFaceCount; psf++, i++ )
+    /*  1  */
+    fc= PS_Encodings;
+    for ( enc= 0; enc < ENCODINGps_COUNT; fc++, enc++ )
 	{
-	if  ( psf->psfEncodingUsed >= 0			&&
-	      psf->psfEncodingUsed < ENCODINGps_COUNT	&&
-	      ! encodingDefined[psf->psfEncodingUsed]	)
+	if  ( pstl->pstlEncodingUsed[enc] )
 	    {
-	    appPsDefineFontEncoding( sos,
-			PS_Encodings[psf->psfEncodingUsed].fcEncodingArray,
-			PS_Encodings[psf->psfEncodingUsed].fcGlyphNames );
-
-	    encodingDefined[psf->psfEncodingUsed]= 1;
+	    sioOutPrintf( sos, "\n" );
+	    utilPsDefineFontEncoding( sos,
+				fc->fcPsEncodingArrayName, fc->fcGlyphNames );
 	    }
 	}
 
-    psf= psfl->psflFaces;
-    for ( i= 0; i < psfl->psflFaceCount; psf++, i++ )
+    /*  2  */
+    psf= (PostScriptFace *)utilTreeGetFirst( pstl->pstlFaceTree, &fontName );
+    while( psf )
 	{
 	const AfmFontInfo *	afi= psf->psfAfi;
-	int			done;
-	PostScriptFace *	psd;
+	char			exceptionSuffix[22];
 
-	if  ( ! psf->psfAppearsInText && ! allFonts )
-	    { continue;	}
+	sioOutPrintf( sos, "\n" );
+	sioOutPrintf( sos, "%%%%%%%%\n" );
+	sioOutPrintf( sos, "%%%%%%%%    %s\n", afi->afiFullName );
+	sioOutPrintf( sos, "%%%%%%%% -> %s\n", afi->afiFontName );
+	sioOutPrintf( sos, "%%%%%%%%\n\n" );
 
-	if  ( ! started )
-	    { sioOutPrintf( sos, "\n" ); started= 1;	}
-
-	psd= psfl->psflFaces;
-	for ( done= 0; done < i; psd++, done++ )
+	if  ( psf->psfAppearsInText || allFonts )
 	    {
-	    if  ( ! psd->psfAppearsInText && ! allFonts )
-		{ continue;	}
+	    const FaceReference *	fr;
+	    int				ref;
+	    const SupportedCharset *	sc;
 
-	    if  ( psd->psfEncodingUsed == psf->psfEncodingUsed	&&
-		  ! strcmp( afi->afiFontName, psd->psfAfi->afiFontName ) )
-		{ break;	}
+	    fc= PS_Encodings;
+	    sc= afi->afiSupportedCharsets;
+	    for ( enc= 0; enc < ENCODINGps_COUNT; sc++, fc++, enc++ )
+		{
+		if  ( ! psf->psfEncodingUsed[enc] )
+		    { continue;	}
+
+		exceptionSuffix[0]= '\0';
+
+		if  ( sc->scNonStandardGlyphNames )
+		    {
+		    int				i;
+		    const char * const *	glyphNames= fc->fcGlyphNames;
+
+		    sprintf( exceptionSuffix, "-x%d", faceNumber );
+
+		    sioOutPrintf( sos, "/%s%s 256 array def\n",
+						fc->fcPsEncodingArrayName,
+						exceptionSuffix );
+		    sioOutPrintf( sos, "%s %s%s copy\n",
+						fc->fcPsEncodingArrayName,
+						fc->fcPsEncodingArrayName,
+						exceptionSuffix );
+
+		    for ( i= 0; i < fc->fcGlyphCount; i++ )
+			{
+			int			glyphno;
+			const AfmCharMetric *	acm;
+
+			if  ( i >= 256 )
+			    { LDEB(i); break;	}
+			if  ( ! glyphNames[i] )
+			    { continue;	}
+
+			glyphno= sc->scCodeToGlyphMapping[i];
+			if  ( glyphno < 0 )
+			    { continue;	}
+			if  ( glyphno >= afi->afiMetricCount )
+			    { LLDEB(glyphno,afi->afiMetricCount); continue; }
+
+			acm= afi->afiMetrics+ glyphno;
+			if  ( strcmp( acm->acmN, glyphNames[i] ) )
+			    {
+			    sioOutPrintf( sos, "  dup %d /%s put\n",
+							i,  acm->acmN );
+			    }
+			}
+
+		    sioOutPrintf( sos, "readonly\n" );
+		    }
+
+		utilPsDefineEncodedFont( sos, afi,
+			    fc->fcPsEncodingSuffix,
+			    fc->fcPsEncodingArrayName,
+			    exceptionSuffix );
+		}
+
+	    fr= psf->psfReferences;
+	    for ( ref= 0; ref < psf->psfReferenceCount; fr++, ref++ )
+		{
+		const char *	encodingSuffix= "";
+
+		if  ( ! fr->frAppearsInText && ! allFonts )
+		    { continue;	}
+
+		exceptionSuffix[0]= '\0';
+
+		if  ( fr->frEncoding >= 0		&&
+		      fr->frEncoding < ENCODINGps_COUNT	)
+		    {
+		    fc= PS_Encodings+ fr->frEncoding;
+		    sc= afi->afiSupportedCharsets+ fr->frEncoding;
+
+		    encodingSuffix= fc->fcPsEncodingSuffix;
+		    if  ( sc->scNonStandardGlyphNames )
+			{ sprintf( exceptionSuffix, "-x%d", faceNumber ); }
+		    }
+
+		sioOutPrintf( sos, "/%s\t{ /%s%s%s exch selectfont } def\n",
+				fr->frFaceId,
+				afi->afiFontName,
+				encodingSuffix,
+				exceptionSuffix );
+		}
 	    }
 
-	if  ( psf->psfEncodingUsed >= 0			&&
-	      psf->psfEncodingUsed < ENCODINGps_COUNT	)
-	    {
-	    encodingSuffix= PS_Encodings[psf->psfEncodingUsed].fcEncodingSuffix;
-	    encodingArray= PS_Encodings[psf->psfEncodingUsed].fcEncodingArray;
-	    }
-
-	if  ( encodingSuffix && done >= i )
-	    {
-	    sioOutPrintf( sos, "/%s findfont dup length dict begin\n",
-						    afi->afiFontName );
-	    sioOutPrintf( sos, "  {\n" );
-	    sioOutPrintf( sos, "    1 index /FID ne\n" );
-	    sioOutPrintf( sos, "      { def } { pop pop } ifelse\n" );
-	    sioOutPrintf( sos, "  } forall\n");
-	    sioOutPrintf( sos, "  /Encoding %s def currentdict\n", encodingArray );
-	    sioOutPrintf( sos, "end " );
-
-	    sioOutPrintf( sos, "/%s%s exch definefont pop\n\n",
-					afi->afiFontName,
-					encodingSuffix?encodingSuffix:"" );
-	    }
-	}
-
-    psf= psfl->psflFaces;
-    for ( i= 0; i < psfl->psflFaceCount; psf++, i++ )
-	{
-	if  ( ! psf->psfAppearsInText && ! allFonts )
-	    { continue;	}
-
-	encodingSuffix= (const char *)0;
-
-	if  ( psf->psfEncodingUsed >= 0			&&
-	      psf->psfEncodingUsed < ENCODINGps_COUNT	)
-	    {
-	    encodingSuffix= PS_Encodings[psf->psfEncodingUsed].fcEncodingSuffix;
-	    }
-
-	sioOutPrintf( sos, "/%s\t{ /%s%s exch selectfont } def\n",
-				psf->psfFontId,
-				psf->psfAfi->afiFontName,
-				encodingSuffix?encodingSuffix:"" );
+	psf= (PostScriptFace *)utilTreeGetNext( pstl->pstlFaceTree,
+								&fontName );
+	faceNumber++;
 	}
 
     sioOutPrintf( sos, "\n" );
@@ -826,6 +1205,8 @@ void appPsDefineEpsProcs(	SimpleOutputStream *		sos )
 void appPsBeginEpsObject(	PrintingState *		ps,
 				int			x0Twips,
 				int			y0Twips,
+				double			xScale,
+				double			yScale,
 				int			llxTwips,
 				int			llyTwips,
 				int			urxTwips,
@@ -837,8 +1218,8 @@ void appPsBeginEpsObject(	PrintingState *		ps,
 
     sioOutPrintf( ps->psSos, "BeginEPSF\n" );
 
-    sioOutPrintf( ps->psSos, "[ %d %d %d %d %d %d ] concat\n",
-				    20, 0, 0, -20, x0Twips, y0Twips );
+    sioOutPrintf( ps->psSos, "[ %g %d %d %g %d %d ] concat\n",
+			20* xScale, 0, 0, -20* yScale, x0Twips, y0Twips );
 
     sioOutPrintf( ps->psSos, "newpath %d %d moveto ",
 					    llxTwips/20, llyTwips/20 );
@@ -851,7 +1232,7 @@ void appPsBeginEpsObject(	PrintingState *		ps,
     sioOutPrintf( ps->psSos, "closepath clip\n" );
 
     sioOutPrintf( ps->psSos, "%%%%BeginDocument: (" );
-    appPsPrintString( ps->psSos, file, strlen( (char *)file ) );
+    utilPsPrintString( ps, file, strlen( (char *)file ) );
     sioOutPrintf( ps->psSos, ")\n" );
 
     return;
@@ -866,6 +1247,25 @@ void appPsEndEpsObject(	PrintingState *		ps )
     return;
     }
 
+/************************************************************************/
+/*									*/
+/*  Fill a rectangle							*/
+/*									*/
+/************************************************************************/
+
+void utilPsFillRectangle(	PrintingState *		ps,
+				int			x,
+				int			y,
+				int			wide,
+				int			high )
+    {
+    sioOutPrintf( ps->psSos, "%d %d %d %d rectfill\n", x, y, wide, high );
+
+    ps->psLastPageMarked= ps->psPagesPrinted;
+
+    return;
+    }
+
 /************************************************************************/
 /*									*/
 /*  Insert the destination of a pdfmark jump in the Printout.		*/
@@ -882,10 +1282,31 @@ int utilPsDestPdfmark(		PrintingState *		ps,
     int			x= 0;
     int			y= lineTop;
 
+    const char *	s;
+    int			i;
+
     top= AT2_Y( x, y, at );
 
-    sioOutPrintf( ps->psSos, "[ /Dest" );
-    sioOutPrintf( ps->psSos, "  /%.*s\n", refSize, refName );
+    sioOutPrintf( ps->psSos, "[ /Dest /" );
+
+    s= refName;
+    for ( i= 0; i < refSize; s++, i++ )
+	{
+	if  ( *s == '('	|| *s == ')'		||
+	      *s == '<'	|| *s == '>'		||
+	      *s == '['	|| *s == ']'		||
+	      *s == '{'	|| *s == '}'		||
+	      *s == '/'	|| *s == '%'		||
+	      isspace( *s ) || ! isascii( *s )	)
+	    {
+	    sioOutPutCharacter( '_', ps->psSos );
+	    continue;
+	    }
+
+	sioOutPutCharacter( *s, ps->psSos );
+	}
+
+    sioOutPrintf( ps->psSos, "\n" );
 
     sioOutPrintf( ps->psSos, "  /View [ /XYZ null %d null ]\n", top );
 
@@ -894,3 +1315,131 @@ int utilPsDestPdfmark(		PrintingState *		ps,
     return 0;
     }
 
+/************************************************************************/
+/*									*/
+/*  Write links to be picked up by the distiller.			*/
+/*									*/
+/************************************************************************/
+
+static void utilPsUriLinkDestination(	PrintingState *	ps )
+    {
+    utilPsPrintString( ps,
+	(const unsigned char *)ps->psLinkFile, ps->psLinkFileSize );
+
+    if  ( ps->psLinkMarkSize > 0 )
+	{
+	sioOutPutCharacter( '#', ps->psSos );
+	utilPsPrintString( ps,
+			    (const unsigned char *)ps->psLinkMark,
+			    ps->psLinkMarkSize );
+	}
+    }
+
+static void utilPsWebLinkDestination(	PrintingState *	ps )
+    {
+    sioOutPrintf( ps->psSos, "  /Action << /Subtype /URI /URI (" );
+
+    utilPsUriLinkDestination( ps );
+
+    sioOutPrintf( ps->psSos, ") >>\n" );
+
+    return;
+    }
+
+static void utilPsFileLinkDestMark(	PrintingState *		ps )
+    {
+    const unsigned char *	file;
+    int				size;
+
+    file= (const unsigned char *)ps->psLinkFile;
+    size= ps->psLinkFileSize;
+
+    if  ( size > 5 && ! strncmp( (const char *)file, "file:", 5 ) )
+	{ file += 5; size -= 5; }
+    else{
+	while( size > 0 && isalpha( *file ) )
+	    { file++; size--;	}
+
+	if  ( size > 0 && *file == ':' )
+	    { utilPsWebLinkDestination( ps ); return; }
+
+	file= (const unsigned char *)ps->psLinkFile;
+	size= ps->psLinkFileSize;
+	}
+
+    sioOutPrintf( ps->psSos, "  /Action /Launch /File (" );
+
+    utilPsPrintString( ps, file, size );
+
+    sioOutPrintf( ps->psSos, ")\n" );
+
+    if  ( ps->psLinkMarkSize )
+	{
+	sioOutPrintf( ps->psSos, "  /URI (" );
+	utilPsUriLinkDestination( ps );
+	sioOutPrintf( ps->psSos, ")\n" );
+	}
+
+    return;
+    }
+
+void utilPsFlushLink(		PrintingState *		ps,
+				int			x0,
+				int			wide,
+				int			lineTop,
+				int			lineHeight )
+    {
+    if  ( ps->psLinkParticulesDone > 0 )
+	{
+	sioOutPrintf( ps->psSos, "[ /Rect [ %d %d %d %d ]\n",
+				    ps->psLinkRectLeft, lineTop+ lineHeight, 
+				    x0+ wide,
+				    lineTop );
+
+	sioOutPrintf( ps->psSos, "  /Border [ 0 0 0 ]\n" );
+
+	if  ( ps->psLinkFileSize == 0 )
+	    {
+	    if  ( ! ps->psLinkMark )
+		{ XDEB(ps->psLinkMark);	}
+	    else{
+		int		i;
+		const char *	s;
+
+		sioOutPrintf( ps->psSos, "  /Dest /" );
+
+		s= ps->psLinkMark;
+		for ( i= 0; i < ps->psLinkMarkSize; s++, i++ )
+		    {
+		    if  ( *s == '('	|| *s == ')'		||
+			  *s == '<'	|| *s == '>'		||
+			  *s == '['	|| *s == ']'		||
+			  *s == '{'	|| *s == '}'		||
+			  *s == '/'	|| *s == '%'		||
+			  isspace( *s ) || ! isascii( *s )	)
+			{
+			sioOutPutCharacter( '_', ps->psSos );
+			continue;
+			}
+
+		    sioOutPutCharacter( *s, ps->psSos );
+		    }
+
+		sioOutPrintf( ps->psSos, "\n" );
+		}
+	    }
+	else{
+	    utilPsFileLinkDestMark( ps );
+	    }
+
+
+	sioOutPrintf( ps->psSos, "  /Subtype /Link\n" );
+	sioOutPrintf( ps->psSos, "/ANN pdfmark\n" );
+
+	ps->psLinkParticulesDone= 0;
+	ps->psLinkRectLeft= x0;
+	}
+
+    return;
+    }
+
