@@ -1,16 +1,84 @@
-#   include	"config.h"
+#   include	"bitmapConfig.h"
 
 #   include	<stdio.h>
 #   include	<stdlib.h>
 #   include	<string.h>
 #   include	"bmintern.h"
-#   include	"bmp.h"
 #   include	<sioHex.h>
 #   include	<sioEndian.h>
 #   include	<sioStdio.h>
 
 #   include	<appDebugon.h>
 
+/************************************************************************/
+/*									*/
+/*  Data structures.							*/
+/*									*/
+/************************************************************************/
+
+/************************************************************************/
+/*  Describes a BMP file.						*/
+/************************************************************************/
+
+#define COMPRESS_RGB        0L      /* No compression               */
+#define COMPRESS_RLE8       1L      /* 8 bits per pixel compression */
+#define COMPRESS_RLE4       2L      /* 4 bits per pixel compression */
+#define BMP_ID              0x4d42  /* BMP "magic" number           */
+
+/************************************************************************/
+/*  Common information for all kinds of BMP files.			*/
+/************************************************************************/
+typedef struct BmpFileHeader
+    {
+			/************************************************/
+			/*  Common to all variants.			*/
+			/************************************************/
+    short	bfhMagic;
+    long	bfhFileSize;
+    short	bfhReserved1;
+    short	bfhReserved2;
+    long	bfhBitmapDataOffset;
+    long	bfhSizeOfRestOfHeader;	/*  It is the Type of file!	*/
+			/************************************************/
+			/*  Common to all variants.			*/
+			/************************************************/
+    long	bfhPixelsWide;
+    long	bfhPixelsHigh;
+    short	bfhPlanes;
+    short	bfhBitsPerPixel;
+			/************************************************/
+			/*  Common to Windows 3.* and OS/2 2.*		*/
+			/************************************************/
+    long	bfhCompression;
+    long	bfhBufferLength;
+    long	bfhXResolution;
+    long	bfhYResolution;
+    long	bfhColorCount;
+    long	bfhImportantColors;
+    } BmpFileHeader;
+
+/************************************************************************/
+/*  Structures desribing ICO files.					*/
+/************************************************************************/
+typedef struct IcoFileHeader
+    {
+    short	ifhReserved;
+    short	ifhType;
+    short	ifhCount;
+    } IcoFileHeader;
+
+typedef struct IcoDirectoryEntry
+    {
+    unsigned char	ideWidth;
+    unsigned char	ideHeight;
+    unsigned char	ideColorCount;
+    unsigned char	ideReserved;
+    short		idePlanes;
+    short		ideBitCount;
+    long		ideBytesInRes;
+    long		ideImageOffset;
+    } IcoDirectoryEntry;
+
 /************************************************************************/
 /*									*/
 /*  Read a 3 or 4 byte color table.					*/
@@ -129,8 +197,8 @@ static int bmpReadImageHeader(	BmpFileHeader *		bfh,
 	    /*  4  */
 	    bfh->bfhCompression= 0;
 	    bfh->bfhBufferLength= 0;
-	    bfh->bfhXResolution= 100* POINTS_PER_CM;
-	    bfh->bfhYResolution= 100* POINTS_PER_CM;
+	    bfh->bfhXResolution= POINTS_PER_M;
+	    bfh->bfhYResolution= POINTS_PER_M;
 	    bfh->bfhColorCount= colorCount;
 	    bfh->bfhImportantColors= colorCount;
 
@@ -354,26 +422,28 @@ static int bmpHeaderToDescription(	BitmapDescription *	bd,
 	else{ bd->bdColorCount= bfh->bfhColorCount; }
 
 	bd->bdColorEncoding= BMcoRGB8PALETTE;
+	bd->bdBitsPerPixel= ( 1+ assumeAlpha )* bfh->bfhBitsPerPixel;
+	}
+    else{
+	bd->bdColorEncoding= BMcoRGB;
 	}
-    else{ bd->bdColorEncoding= BMcoRGB;		}
 
     bd->bdHasAlpha= assumeAlpha;
 
-    bd->bdBitsPerPixel= ( 1+ assumeAlpha )* bfh->bfhBitsPerPixel;
-    bd->bdBytesPerRow= ( bd->bdBitsPerPixel* bd->bdPixelsWide + 7 )/ 8;
-    bd->bdBufferLength= bd->bdBytesPerRow* bd->bdPixelsHigh;
+    if  ( bmCalculateSizes( bd ) )
+	{ LDEB(1); return -1;	}
 
     *pBytesPerRow= ( bfh->bfhBitsPerPixel* bd->bdPixelsWide + 7 )/ 8;
 
-    bd->bdUnit= BMunCM;
-    bd->bdXResolution= bfh->bfhXResolution/100;
-    bd->bdYResolution= bfh->bfhYResolution/100;
+    bd->bdUnit= BMunM;
+    bd->bdXResolution= bfh->bfhXResolution;
+    bd->bdYResolution= bfh->bfhYResolution;
 
     /********************************************************************/
     /*  Check people that even run microsoft systems in their brain.	*/
     /********************************************************************/
-    if  ( bd->bdXResolution == 0 || bd->bdXResolution == 0 )
-	{ bd->bdYResolution= bd->bdXResolution= 1; bd->bdUnit= BMunPOINT; }
+    if  ( bd->bdXResolution < 500 || bd->bdXResolution < 500 )
+	{ bd->bdYResolution= bd->bdXResolution= 1; bd->bdUnit= BMunPIXEL; }
 
     return 0;
     }
@@ -411,18 +481,26 @@ static int bmbmpDescriptionToHeader(	BmpFileHeader *		bfh,
 
     switch( bd->bdUnit )
 	{
-	case BMunCM:
-	    bfh->bfhXResolution= 100* bd->bdXResolution;
-	    bfh->bfhYResolution= 100* bd->bdYResolution;
+	case BMunM:
+	    bfh->bfhXResolution= bd->bdXResolution;
+	    bfh->bfhYResolution= bd->bdYResolution;
 	    break;
+
 	case BMunINCH:
 	    bfh->bfhXResolution= 40* bd->bdXResolution;
 	    bfh->bfhYResolution= 40* bd->bdYResolution;
 	    break;
+
 	case BMunPOINT:
-	    bfh->bfhXResolution= 100* POINTS_PER_CM* bd->bdXResolution;
-	    bfh->bfhYResolution= 100* POINTS_PER_CM* bd->bdYResolution;
+	    bfh->bfhXResolution= POINTS_PER_M* bd->bdXResolution;
+	    bfh->bfhYResolution= POINTS_PER_M* bd->bdYResolution;
 	    break;
+
+	case BMunPIXEL:
+	    bfh->bfhXResolution= 0;
+	    bfh->bfhYResolution= 0;
+	    break;
+
 	default:
 	    LDEB(bd->bdUnit); return -1;
 	}
@@ -721,14 +799,15 @@ static int bmBmpReadImageBytes(	unsigned char **		pBuffer,
 /*									*/
 /*  1)	Read and check header.						*/
 /*  2)  Translate header to a BitmapDescription.			*/
-/*  3)  Seek to data. ( We are supposed to be there).			*/
+/*  3)  Seek to data. ( We are supposed to be there ).			*/
 /*									*/
 /************************************************************************/
+
 int bmReadBmpFile(	const char *		filename,
 			unsigned char **	pBuffer,
 			BitmapDescription *	bd,
 			int *			pPrivateFormat,
-			double *		pCompressionFactor	)
+			double *		pCompressionFactor )
     {
     SimpleInputStream *	sis;
     BmpFileHeader	bfh;
@@ -1250,13 +1329,20 @@ int bmReadIcoFile(	const char *		filename,
 			int *			pPrivateFormat,
 			double *		pCompressionFactor	)
     {
-    SimpleInputStream *	sis;
-    BmpFileHeader	bfh;
-    IcoFileHeader	ifh;
-    IcoDirectoryEntry	ide;
-    int			scanLine;
-    RGB8Color *		palette= (RGB8Color *)0;
-    unsigned char *	buffer;
+    int				rval= 0;
+    SimpleInputStream *		sis= (SimpleInputStream *)0;
+    unsigned char *		buffer= (unsigned char *)0;
+    RGB8Color *			palette= (RGB8Color *)0;
+
+    BmpFileHeader		bfh;
+    IcoFileHeader		ifh;
+    IcoDirectoryEntry		ide;
+
+    int				alphaByteCount;
+    unsigned char *		ab= (unsigned char *)0;
+    unsigned char *		rb= (unsigned char *)0;
+
+    int				row;
 
     unsigned char *	to;
     int			done;
@@ -1265,15 +1351,15 @@ int bmReadIcoFile(	const char *		filename,
 
     sis= sioInStdioOpen( filename );
     if  ( ! sis )
-	{ SDEB(filename); return -1;	}
+	{ SDEB(filename); rval= -1; goto ready;	}
 
     ifh.ifhReserved= sioEndianGetLeInt16( sis );
     if  ( ifh.ifhReserved != 0 )
-	{ LDEB(ifh.ifhReserved); return -1;	}
+	{ LDEB(ifh.ifhReserved); rval= -1; goto ready;	}
 
     ifh.ifhType= sioEndianGetLeInt16( sis );
     if  ( ifh.ifhType != 1 )
-	{ LDEB(ifh.ifhType); return -1;	}
+	{ LDEB(ifh.ifhType); rval= -1; goto ready;	}
 
     ifh.ifhCount= sioEndianGetLeInt16( sis );
     if  ( ifh.ifhCount != 1 )
@@ -1285,7 +1371,7 @@ int bmReadIcoFile(	const char *		filename,
 
     ide.ideReserved= sioInGetCharacter( sis );
     if  ( ide.ideReserved != 0 )
-	{ LDEB(ide.ideReserved); return -1;	}
+	{ LDEB(ide.ideReserved); rval= -1; goto ready;	}
 
     ide.idePlanes= sioEndianGetLeInt16( sis );
     ide.ideBitCount= sioEndianGetLeInt16( sis );
@@ -1297,7 +1383,7 @@ int bmReadIcoFile(	const char *		filename,
     while( bytesRead < ide.ideImageOffset )
 	{
 	if  ( sioInGetCharacter( sis ) < 0 )
-	    { LDEB(ide.ideImageOffset); return -1;	}
+	    { LDEB(ide.ideImageOffset); rval= -1; goto ready;	}
 
 	bytesRead++;
 	}
@@ -1305,12 +1391,12 @@ int bmReadIcoFile(	const char *		filename,
     /*  1  */
     done= bmpReadImageHeader( &bfh, sis, &palette );
     if  ( done < 0 )
-	{ SLDEB(filename,done); sioInClose( sis ); return -1;	}
+	{ SLDEB(filename,done); rval= -1; goto ready;	}
 
     /*  2  */
     if  ( bmpHeaderToDescription( bd, &bytesPerDataRow, pPrivateFormat,
 								&bfh, 1 ) )
-	{ LDEB(1); return -1;	}
+	{ LDEB(1); rval= -1; goto ready;	}
     /*  2a  */
     bd->bdPixelsHigh /= 2;
 
@@ -1318,49 +1404,55 @@ int bmReadIcoFile(	const char *		filename,
 
     buffer= malloc( bd->bdBufferLength+ 3 );
     if  ( ! buffer )
-	{
-	LLDEB(bd->bdBufferLength,buffer);
-
-	if  ( palette )
-	    { free( palette );	}
-
-	sioInClose( sis ); return -1;
-	}
+	{ LLDEB(bd->bdBufferLength,buffer); rval= -1; goto ready; }
 
     /*  4  */
-    for ( scanLine= 0; scanLine < bd->bdPixelsHigh; scanLine++ )
+    for ( row= 0; row < bd->bdPixelsHigh; row++ )
 	{
-	to= buffer+ ( bd->bdPixelsHigh- scanLine- 1 )* bd->bdBytesPerRow;
+	to= buffer+ ( bd->bdPixelsHigh- row- 1 )* bd->bdBytesPerRow;
 
 	done= sioInReadBytes( sis, to, bytesPerDataRow );
 	if  ( done != bytesPerDataRow )
 	    {
 	    LLDEB(done,bytesPerDataRow);
-	    LLDEB(scanLine,bd->bdPixelsHigh); LLDEB(bfh.bfhCompression,done);
-	    sioInClose( sis ); return -1;
+	    LLDEB(row,bd->bdPixelsHigh); LLDEB(bfh.bfhCompression,done);
+	    rval= -1; goto ready;
 	    }
 	}
 
+    alphaByteCount= ( bd->bdPixelsWide+ 7 )/8;
+    ab= malloc( alphaByteCount );
+    if  ( ! ab )
+	{ LXDEB(alphaByteCount,ab); rval= -1; goto ready; }
+    rb= malloc( bd->bdBytesPerRow );
+    if  ( ! rb )
+	{ LXDEB(rb,bd->bdBytesPerRow); rval= -1; goto ready; }
+
+    if  ( bytesPerDataRow > bd->bdBytesPerRow )
+	{
+	LLDEB(bytesPerDataRow,bd->bdBytesPerRow);
+	rval= -1; goto ready;
+	}
+
     /*  5  */
-    for ( scanLine= 0; scanLine < bd->bdPixelsHigh; scanLine++ )
+    for ( row= 0; row < bd->bdPixelsHigh; row++ )
 	{
-	unsigned char	a[8];
-	unsigned char	b[32];
-	int		byteCount= ( bd->bdPixelsWide+ 7 )/8;
 	int		i;
+	int		got;
 
-	to= buffer+ ( bd->bdPixelsHigh- scanLine- 1 )* bd->bdBytesPerRow;
+	to= buffer+ ( bd->bdPixelsHigh- row- 1 )* bd->bdBytesPerRow;
 
-	if  ( sioInReadBytes( sis, a, byteCount ) != byteCount )
+	got= sioInReadBytes( sis, ab, alphaByteCount );
+	if  ( got != alphaByteCount )
 	    {
-	    LLDEB(scanLine,bd->bdPixelsWide);
-	    sioInClose( sis ); return -1;
+	    LLLDEB(row,got,bd->bdPixelsWide);
+	    rval= -1; goto ready;
 	    }
 
-	memcpy( b, to, bytesPerDataRow );
+	memcpy( rb, to, bytesPerDataRow );
 
-	for ( i= 0; i < byteCount; i++ )
-	    { a[i]= ~a[i];	}
+	for ( i= 0; i < alphaByteCount; i++ )
+	    { ab[i]= ~ab[i];	}
 
 	switch( bd->bdColorCount )
 	    {
@@ -1368,50 +1460,88 @@ int bmReadIcoFile(	const char *		filename,
 		for ( i= 0; i < bytesPerDataRow; i++ )
 		    {
 		    *to= 0;
-		    *to |= ( b[i] & 0x80 )      | ( a[i] & 0x80 ) >> 1;
-		    *to |= ( b[i] & 0x40 ) >> 1 | ( a[i] & 0x40 ) >> 2;
-		    *to |= ( b[i] & 0x20 ) >> 2 | ( a[i] & 0x20 ) >> 3;
-		    *to |= ( b[i] & 0x10 ) >> 3 | ( a[i] & 0x10 ) >> 4;
+		    *to |= ( rb[i] & 0x80 )      | ( ab[i] & 0x80 ) >> 1;
+		    *to |= ( rb[i] & 0x40 ) >> 1 | ( ab[i] & 0x40 ) >> 2;
+		    *to |= ( rb[i] & 0x20 ) >> 2 | ( ab[i] & 0x20 ) >> 3;
+		    *to |= ( rb[i] & 0x10 ) >> 3 | ( ab[i] & 0x10 ) >> 4;
 		    *(++to)= 0;
-		    *to |= ( b[i] & 0x08 ) << 4 | ( a[i] & 0x08 ) << 3;
-		    *to |= ( b[i] & 0x04 ) << 3 | ( a[i] & 0x04 ) << 2;
-		    *to |= ( b[i] & 0x02 ) << 2 | ( a[i] & 0x02 ) << 1;
-		    *to |= ( b[i] & 0x01 ) << 1 | ( a[i] & 0x01 )     ;
+		    *to |= ( rb[i] & 0x08 ) << 4 | ( ab[i] & 0x08 ) << 3;
+		    *to |= ( rb[i] & 0x04 ) << 3 | ( ab[i] & 0x04 ) << 2;
+		    *to |= ( rb[i] & 0x02 ) << 2 | ( ab[i] & 0x02 ) << 1;
+		    *to |= ( rb[i] & 0x01 ) << 1 | ( ab[i] & 0x01 )     ;
 		    to++;
 		    }
 		break;
+
 	    case 16:
 		for ( i= 0; i < bd->bdPixelsWide; i += 2 )
 		    {
-		    if  ( ( a[i/8] << (i%8) ) & 0x80 )
-			{ *(to++)= ( b[i/2] & 0xf0 ) | 0x0f;	}
+		    if  ( ( ab[i/8] << (i%8) ) & 0x80 )
+			{ *(to++)= ( rb[i/2] & 0xf0 ) | 0x0f;	}
 		    else{ *(to++)= 0x00;		}
 
-		    if  ( ( a[i/8] << (i%8) ) & 0x40 )
-			{ *(to++)=   ( b[i/2] << 4 ) | 0x0f;	}
+		    if  ( ( ab[i/8] << (i%8) ) & 0x40 )
+			{ *(to++)=   ( rb[i/2] << 4 ) | 0x0f;	}
 		    else{ *(to++)= 0x00;			}
 		    }
 		break;
+
+	    case 256:
+		for ( i= 0; i < bd->bdPixelsWide; i++ )
+		    {
+		    *(to++)= rb[i];
+		    if  ( ( ab[i/8] << (i%8) ) & 0x80 )
+			{ *(to++)= 255;	}
+		    else{ *(to++)= 0;	}
+		    }
+		break;
+
+	    case 0:
+		for ( i= 0; i < bd->bdPixelsWide; i++ )
+		    {
+		    *(to++)= rb[3* i+ 0];
+		    *(to++)= rb[3* i+ 1];
+		    *(to++)= rb[3* i+ 2];
+		    if  ( ( ab[i/8] << (i%8) ) & 0x80 )
+			{ *(to++)= 255;	}
+		    else{ *(to++)= 0;	}
+		    }
+		break;
+
 	    case 4:
 	    case 8:
 	    default:
-		LDEB(bd->bdColorCount); sioInClose( sis ); return -1;
+		LDEB(bd->bdColorCount); rval= -1; goto ready;
 	    }
 	}
 
-    sioInClose( sis );
-
-    *pBuffer= buffer;
+    *pBuffer= buffer; buffer= (unsigned char *)0; /* steal */
     *pPrivateFormat= bfh.bfhSizeOfRestOfHeader;
 
     if  ( bfh.bfhColorCount > 0				&&
 	  ! bmMakeMonochrome( bd, palette, buffer )	)
-	{ return 0;	}
+	{ palette= (RGB8Color *)0; /* freed */	}
+    else{
+	if  ( bfh.bfhBitsPerPixel != 24 || bfh.bfhColorCount != 0 )
+	    {
+	    bd->bdRGB8Palette= palette; palette= (RGB8Color *)0; /* steal */
+	    }
+	}
 
-    if  ( bfh.bfhBitsPerPixel != 24 || bfh.bfhColorCount != 0 )
-	{ bd->bdRGB8Palette= palette;	}
+  ready:
 
-    return 0;
+    if  ( palette )
+	{ free( palette );	}
+    if  ( rb )
+	{ free( rb );	}
+    if  ( ab )
+	{ free( ab );	}
+    if  ( buffer )
+	{ free( buffer );	}
+    if  ( sis )
+	{ sioInClose( sis );	}
+
+    return rval;
     }
 
 /************************************************************************/
